/**
 * @file
 * VuoList_VuoMidiInputDevice C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoMidiInputDevice_H
#define VuoList_VuoMidiInputDevice_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoMidiInputDevice VuoList_VuoMidiInputDevice
 * A list of @ref VuoMidiInputDevice elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoMidiInputDevice_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoMidiInputDevice elements.
 */
typedef const struct VuoList_VuoMidiInputDevice_struct { void *l; } * VuoList_VuoMidiInputDevice;
/// @}
#endif

/**
 * Creates a new list of @ref VuoMidiInputDevice elements.
 */
VuoList_VuoMidiInputDevice VuoListCreate_VuoMidiInputDevice(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoMidiInputDevice to quickly initialize a large list.
 */
VuoList_VuoMidiInputDevice VuoListCreateWithCount_VuoMidiInputDevice(const unsigned long count, const VuoMidiInputDevice value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoMidiInputDevice VuoListCopy_VuoMidiInputDevice(const VuoList_VuoMidiInputDevice list);

/**
 * Returns the @ref VuoMidiInputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoMidiInputDevice VuoListGetValue_VuoMidiInputDevice(const VuoList_VuoMidiInputDevice list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoMidiInputDevice *VuoListGetData_VuoMidiInputDevice(const VuoList_VuoMidiInputDevice list);

/**
 * Changes the @ref VuoMidiInputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoMidiInputDevice(const VuoList_VuoMidiInputDevice list, const VuoMidiInputDevice value, const unsigned long index);

/**
 * Inserts the `VuoMidiInputDevice` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoMidiInputDevice(const VuoList_VuoMidiInputDevice list, const VuoMidiInputDevice value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list, const VuoMidiInputDevice value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list, const VuoMidiInputDevice value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoMidiInputDevice_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoMidiInputDevice_areEqual(const VuoList_VuoMidiInputDevice a, const VuoList_VuoMidiInputDevice b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoMidiInputDevice_isLessThan(const VuoList_VuoMidiInputDevice a, const VuoList_VuoMidiInputDevice b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list);

/**
 * Removes the `VuoMidiInputDevice` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoMidiInputDevice(VuoList_VuoMidiInputDevice list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoMidiInputDevice(const VuoList_VuoMidiInputDevice list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoMidiInputDevice VuoList_VuoMidiInputDevice_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoMidiInputDevice_getJson(const VuoList_VuoMidiInputDevice value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoMidiInputDevice_getSummary(const VuoList_VuoMidiInputDevice value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoMidiInputDevice VuoList_VuoMidiInputDevice_makeFromString(const char *str);
char * VuoList_VuoMidiInputDevice_getString(const VuoList_VuoMidiInputDevice value);
/// @}

/**
 * @}
 */

#endif
